# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: install.py 783 2010-01-09 14:28:06Z icemac $
"""Initial generation."""


def evolve(context):
    # nothing to install initially
    pass
