# -*- coding: latin-1 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt
# $Id: evolve5.py 783 2010-01-09 14:28:06Z icemac $

__docformat__ = "reStructuredText"


generation = 5


def evolve(context):
    """Failed approach to install the importer."""
    pass
