# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: i18n.py 1009 2010-08-15 14:20:55Z icemac $

"""I18n message factory for icemac.addressbook. """

import zope.i18nmessageid

_ = MessageFactory = zope.i18nmessageid.MessageFactory('icemac.addressbook')
