div#content label span.required {
    color: red;
}

.required-info {
    font-size: small;
}

.required-info .required {
    color: red;
}

div#content form div.widget {
    margin-left: 1em;
}

div#content fieldset {
    background-color: #f0f0d0;
    opacity: 0.92;
    padding: 0 4px 6px 4px;
    border: 2px solid #cccccc;
    border-radius: 5px;
    -moz-border-radius: 5px;
}

div#content fieldset fieldset {
    background-color: #f0f0e0;
}

div#content legend {
    background-color: #f0f0d0;
    font-weight: bold;
    padding: 0 2px;
    margin-top: 3px;
    border-radius: 5px;
    -moz-border-radius: 5px;
}

div#content fieldset fieldset {
    float: right;
}

div#content fieldset fieldset div.row {
    background-color: #f0f0e0;
    font-weight: normal;
    font-size: small;
}

div#content fieldset fieldset legend {
    font-size: small;
}

div#content select {
    opacity: 0.85;
}

div#content textarea {
    height: 15ex;
    width: 100%;
    font-size: 11px;
}

/* buttons */
form div.buttons {
    clear: left;
    position: fixed;
    bottom: 0;
    left: 0;
    background-color: lightGrey;
    opacity: 0.85;
    padding: 5px 0 3px 10px;
    width: 100%;
}

form div.widget div.buttons {
    clear: inherit;
    position: inherit;
    background-color: inherit;
    opacity: inherit;
    padding: inherit;
    width: inherit;
}

div#content form div {
    clear: both;
}

form div.buttons input {
    opacity: 1;
}


/* for forms: (metadata top right) */

form#edit-form div.group {
    float: left;
}

form#edit-form div.group + fieldset {
    float: right;
}

form div.group + fieldset + fieldset{
    clear: left;
}

form#search-export-form div.buttons {
    background-color: transparent;
    left: 100px;
}

form#search-export-form div.buttons input#form-buttons-delete {
    margin-left: 25px;
}